//
//  UMCrypto.h
//  ulib
//
//  Created by Andreas Fink on 10.12.08.
//  Copyright (c) Andreas Fink (andreas@fink.org), Basel, Switzerland. All rights reserved
//


#import "UMObject.h"

#define DES_BLOCK_SIZE 64
#define DES_SALT_LEN 56
#define DES_KEY_LEN 56

#define DES3_BLOCK_SIZE 64
#define DES3_KEY_LEN 168
#define DES3_SALT_LEN 56

#define CAST5_BLOCK_SIZE 64

#define RSA_KEY_LEN 4096
#define RSA_EXPONENT 65537
#define RSA_PADDING_LEN 41

#if 0
#ifdef __APPLE__
#define HAS_KEYCHAIN_UTILITIES  1
#define HAS_COMMON_CRYPTO       1
#undef HAS_OPENSSL
#endif

#ifdef LINUX
#undef HAS_KEYCHAIN_UTILITIES
#undef HAS_COMMON_CRYPTO
#undef __APPLE__
#define HAS_OPENSSL             1
#endif
#endif

#define HAS_OPENSSL     1


@interface UMCrypto : UMObject
{
	NSInteger	        enable;
	NSInteger	        pos;
	NSInteger	        method;
	const char          *cryptoVector;
	NSInteger	        vectorSize;
    NSData              *deskey;              /* For symmetric cryptography*/
    NSData              *cryptorKey;
    unsigned char       *salt;
    NSData              *iv;
    NSData              *publicKey;           /* For unsymmetric cryptography, having PEM format*/
    NSData              *privateKey;          /* Ditto */
    int                 public_keylen;
    int                 private_keylen;
    NSData              *cryptorPublicKey;
    NSData              *cryptorPrivateKey;
}

@property(readwrite,assign)		NSInteger	  enable;
@property(readwrite,assign)		NSInteger	  pos;
@property(readwrite,assign)		NSInteger	  method;
@property(readwrite,assign)		const char    *cryptoVector;
@property(readwrite,assign)		NSInteger	  vectorSize;
@property(readwrite,assign)		unsigned char *salt;
@property(readwrite,strong)		NSData        *iv;
@property(readwrite,strong)		NSData        *publicKey;
@property(readwrite,strong)		NSData        *privateKey;
@property(readwrite,strong)		NSData        *deskey;
@property(readwrite,strong)		NSData        *cryptorKey;

#pragma mark -
#pragma mark Initialisation
+ (UMCrypto *)clearTextVector;
- (UMCrypto *)initWithVector:(const char *)vector size:(NSInteger)len;
- (UMCrypto *)initWithKey;
- (void)enableCrypto;
- (void)disableCrypto;



#pragma mark -
#pragma mark Generic Read/Write IO

- (size_t)readBytes:(unsigned char *)bytes
             length:(size_t)length
       fromFileDesc:(int)io
          errorCode:(int *)eno;

- (size_t)writeByte:(unsigned char)byte
         toFileDesc:(int)o
          errorCode:(int *)eno;

- (size_t)writeBytes:(const unsigned char *)bytes
              length:(size_t)length
          toFileDesc:(int)o
           errorCode:(int *)eno;


#pragma mark -
#pragma mark Encryption


#ifdef HAS_COMMON_CRYPTO
- (UMCrypto *)initPublicCrypto;
#endif


- (void)setSeed:(NSInteger)seed;

#ifdef HAS_COMMON_CRYPTO
- (const char *)cryptErrorString:(int)code;
#endif

#pragma mark -
#pragma mark Keychain Utilities
#ifdef HAS_KEYCHAIN_UTILITIES
- (NSData *)dataFromRef:(SecKeyRef)keyRef;
- (SecKeyRef)refFromData:(NSData *)data isPublicKey:(BOOL)isPublic;
#endif

#pragma mark -
#pragma mark DES

- (NSData *)randomDataOfLength:(size_t)length;

#ifdef HAS_OPENSSL
- (NSData *)SSLRandomDataOfLength:(size_t)length;
- (UMCrypto *)initDESInitWithSaltAndIV;
- (UMCrypto *)initDESInitWithKeyWithEntropySource:(NSString *)file withGrade:(int)grade;
- (UMCrypto *)initSSLPublicCryptoWithEntropySource:(NSString *)file;
#endif

#ifdef HAS_COMMON_CRYPTO
- (NSData *)DES3KeyForPassword:(NSData *)password withGrade:(int)grade;
- (NSData *)RSAEncryptWithPlaintextPublic:(NSData *)plaintext;
- (NSData *)RSADecryptWithCiphertextPrivate:(NSData *)ciphertext;
- (NSData *)encryptData:(NSData *)data withPassword:(NSData *)password withKey:(NSData **)key withGrade:(int)grade;
- (NSData *)encryptData:(NSData *)data withPassword:(NSData *)password;
- (NSData *)decryptData:(NSData *)data withPassword:(NSData *)key;
#endif


#ifdef HAS_OPENSSL
- (NSData *)DESEncryptWithPlaintext:(NSData *)plaintext havingLength:(int *)len withPassword:(NSData *)password withKey:(NSData **)key withGrade:(int)grade;
- (NSData *)DESEncryptWithPlaintext:(NSData *)plaintext havingLength:(int *)len withPassword:(NSData *)password;
- (NSData *)DESDecryptWithCiphertext:(NSData *)ciphertext havingLength:(int *)len withKey:(NSData *)key;

- (NSData *)RSAEncryptWithPlaintextSSLPublic:(NSData *)plaintext;
- (NSData *)RSADecryptWithCiphertextSSLPrivate:(NSData *)ciphertext;
- (NSData *)RSAEncryptWithPlaintextSSLPrivate:(NSData *)plaintext;
- (NSData *)RSADecryptWithCiphertextSSLPublic:(NSData *)ciphertext;
- (NSData *)RC4DecryptWithCiphertext:(NSData *)ciphertext havingLength:(int *)len withKey:(NSData *)key;
- (NSData *)DES3DecryptWithCiphertext:(NSData *)ciphertext havingLength:(int *)len withKey:(NSData *)key;
- (NSData *)CAST5DecryptWithCiphertext:(NSData *)ciphertext havingLength:(int *)len withKey:(NSData *)key;
#endif

#ifdef HAS_OPENSSL
- (NSData *)decryptDataWithSSL:(NSData *)data withKey:(NSData *)key;
#endif

@end
